/* -*- coding: utf-8 -*-
 * vim: et:sw=4:sts=4
 */
/**
 * Copyright(c) 2019 Norwegian University of Science and Technology
 *
 */
const prometheus = require('prom-client');
const HttpStatus = require('http-status');
const url = require('url');

const collectDefaultMetrics = prometheus.collectDefaultMetrics;

// Probe every 5th second.
collectDefaultMetrics({timeout: 5000});

const httpRequestDurationMS = new prometheus.Histogram({
    name: 'http_request_duration_ms',
    help: 'Duration of HTTP requests in ms',
    labelNames: ['route'],
    buckets: [1, 2, 5, 10, 15, 30, 50, 75, 100, 150, 200, 300, 400, 500, 600, 700, 800, 900, 1000, 1250, 1500, 1750,
        2000, 2500, 3000, 4000, 5000, 6000, 7500, 10000, 20000, 30000, 40000, 50000, 60000, 70000, 80000, 90000,
        100000, 110000, 120000, 130000, 140000, 150000, 160000, 170000, 180000, 190000, 200000, 210000, 220000, 230000,
        240000, 250000, 260000, 270000, 280000, 290000, 300000]
});

const httpRequestsCounter = new prometheus.Counter({
    name: 'http_request_counter',
    help: 'Number of http requests',
    labelNames: ['status'],
});

const concurrentRequests = new prometheus.Gauge({
    name: 'concurrent_requests',
    help: 'Number of concurrent requests'
});

const neo4jRequestDurationMS = new prometheus.Histogram({
    name: 'neo4j_request_duration_ms',
    help: 'Duration of Neo4J requests in ms',
    buckets: [1, 2, 5, 10, 15, 30, 50, 75, 100, 150, 200, 300, 400, 500, 600, 700, 800, 900, 1000, 1250, 1500, 1750,
        2000, 2500, 3000, 4000, 5000, 6000, 7500, 10000, 20000, 30000, 40000, 50000, 60000, 70000, 80000, 90000,
        100000, 110000, 120000, 130000, 140000, 150000, 160000, 170000, 180000, 190000, 200000, 210000, 220000, 230000,
        240000, 250000, 260000, 270000, 280000, 290000, 300000]
});

const neo4jErrorCount = new prometheus.Counter({
    name: 'neo4j_errors',
    help: 'Number of Neo4J errors'
});

const metricsCallback = (req, res) => {
    res.set('Content-Type', prometheus.register.contentType);
    res.end(prometheus.register.metrics());
};

const elapsedMillis = (start) => {
    return Number((process.hrtime.bigint() - start) / 1000000n);
};

const standardHttpMetricsMiddleware = (req, res, next) => {
    let route = url.parse(req.url).pathname;
    const start = process.hrtime.bigint();

    if (route !== '/favicon.ico' && route !== '/metrics') {
        concurrentRequests.inc();
    }

    res.on('finish', () => {
        if (route !== '/favicon.ico' && route !== '/metrics') {
            let status = HttpStatus.INTERNAL_SERVER_ERROR;
            if (res.statusCode) {
                status = res.statusCode;
            }
            const paramLen = Object.keys(res.req.params).length;
            if (paramLen > 0) {
                route = route.split('/').slice(0, -paramLen).join('/');
            } else if (status > 399 && status < 500) {
                // Middleware have denied access,- just record the first part of the route for Prometheus.
                route = route.split('/').slice(0, 2).join('/');
            }
            route = route.replace(/\/$/, '');
            httpRequestDurationMS.observe({'route': route}, elapsedMillis(start));
            httpRequestsCounter.inc({'status': status});
            concurrentRequests.dec();
        }
    });
    next();
};

module.exports = {
    prometheus,
    httpRequestDurationMS,
    httpRequestsCounter,
    concurrentRequests,
    neo4jRequestDurationMS,
    neo4jErrorCount,
    metricsCallback,
    elapsedMillis,
    standardHttpMetricsMiddleware
};
